#!/usr/bin/perl

package Cognition::Export::Alarm;
use Cognition::Export;
use Cognition::Export::Contact;
use MIME::Base64;
use URI;

sub is
{
	my $subject = shift;
	return unless ($subject);
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://www.w3.org/2002/12/cal/ical#Valarm')
			||  ($type eq 'urn:ietf:rfc:2445#valarm'));
	}
	return;
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		}
	];
}

sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $triples = shift;
	my $subject = shift;
	my $S = $triples->{$subject};
	my @rv = (['BEGIN','VALARM']);

	# UID - I don't think this is actually allowed, but what the hey?!
	my $uid = firstOf($S, 'urn:ietf:rfc:2445#uid',
		'http://www.w3.org/2002/12/cal/ical#uid',
		'http://purl.org/dc/terms/identifier',
		'http://purl.org/dc/elements/1.1/identifier');
	push @rv, ['UID', $uid]
		if ($uid);
	
	# Summary
	my $summary = firstOf($S, 'urn:ietf:rfc:2445#summary',
		'http://www.w3.org/2002/12/cal/ical#summary',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title');
	push @rv, ['SUMMARY', $summary]
		if ($summary);
	
	# Trigger
	my $trigger = firstOf($S, 'urn:ietf:rfc:2445#trigger',
		'http://www.w3.org/2002/12/cal/ical#trigger');
	push @rv, ['TRIGGER', $trigger]
		if ($trigger);
	
	# Action
	my $action = firstOf($S, 'urn:ietf:rfc:2445#action',
		'http://www.w3.org/2002/12/cal/ical#action');
	push @rv, ['ACTION', uc($action)]
		if ($action);
	
	# Description
	my $desc = firstOf($S, 'urn:ietf:rfc:2445#description',
		'http://www.w3.org/2002/12/cal/ical#description',
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description',
		'http://purl.org/dc/terms/abstract');
	push @rv, ['DESCRIPTION', $desc]
		if ($desc);
	
	# Duration
	my $duration = firstOf($S, 'urn:ietf:rfc:2445#duration',
		'http://www.w3.org/2002/12/cal/ical#duration');
	push @rv, ['DURATION', $duration]
		if ($duration);
	
	# Repeat
	my $repeat = firstOf($S, 'urn:ietf:rfc:2445#repeat',
		'http://www.w3.org/2002/12/cal/ical#repeat');
	push @rv, ['REPEAT', sprintf('%d', $repeat)]
		if ($repeat);
	
	# Attach
	my @attachments = allOf($S, 'urn:ietf:rfc:2445#attach',
		'http://www.w3.org/2002/12/cal/ical#attach',
		'http://purl.org/dc/terms/hasPart');
	foreach my $att (@attachments)
	{
		if ($triples->{$att})
		{
			my $E = $triples->{$att};
			my $e = $E->firstVal('http://microformats.org/wiki/rel-enclosure#enclosure');
			push @rv, ['ATTACH;VALUE=uri',$e] if ($e);
		}
		else
		{
			push @rv, ['ATTACH;VALUE=uri',$att];
		}
	}

	# Attendee
	my @agents = allOfObj($S, 'urn:ietf:rfc:2445#attendee',
		'http://www.w3.org/2002/12/cal/ical#attendee');
	foreach my $agent (@agents)
	{
		if ($agent->type && $triples->{$agent->toString})
		{
			my $agentSubject = $triples->{$agent->toString};
			if (Cognition::Export::Contact::is($agentSubject))
			{
				my $a = Cognition::Export::Contact::outVCF($triples, $agent->toString);
				push @rv, ['ATTENDEE',$a];
			}
			else
			{
				my $a = firstOf($agentSubject,
					'urn:ietf:rfc:2426#fn',
					'http://www.w3.org/2006/vcard/ns#fn',
					'http://www.w3.org/2006/03/hcard#fn',
					'http://www.w3.org/2001/vcard-rdf/3.0#fn',
					'http://xmlns.com/foaf/0.1/fullName',
					'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
					'http://purl.org/dc/terms/title',
					'http://purl.org/dc/elements/1.1/title');
				push @rv, ['ATTENDEE;VALUE=text',$a] if ($a);
			}
		}
		else
		{
			my $a = $agent->toString;
			$a =~ s/(^\s+|\s+$)//g;
			$a =~ s/\s+/ /g;
			push @rv, ['ATTENDEE;VALUE=text',$a] if ($a);
		}
	}
		
	# Done.
	push @rv, ['END','VALARM'];
	
	# Convert to text/directory syntax.
	my $rv = '';
	foreach my $line (@rv)
	{
		my ($k, $v, $soft) = @{$line};
		
		if ($k =~ /VALUE.uri$/i && $v =~ /^data\:/)
		{
			my $uri = URI->new($v);
			$v = encode_base64($uri->data, '');
			$k =~ s/VALUE.uri$/ENCODING\=b/i;
		}
		
		$v =~ s/\\/\\\\/g;
		$v =~ s/\r?\n/\\n/g;
		unless ($soft)
		{
			$v =~ s/\;/\\\;/g;
			$v =~ s/\,/\\\,/g;
		}
		
		$rv .= "$k:$v\r\n";
	}
	return $rv;
}

1;
