#!/usr/bin/perl

package Cognition::Export::Event;
use Cognition::Export;
use Cognition::Export::Alarm;
use Cognition::Export::Contact;
use MIME::Base64;
use URI;

sub is
{
	my $subject = shift;
	return unless ($subject);
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://www.w3.org/2002/12/cal/ical#Vevent')
			||  ($type eq 'urn:ietf:rfc:2445#vevent'));
	}
	
	if (firstOf($subject,
		'http://purl.org/rss/1.0/modules/event/startdate',
		'http://purl.org/rss/1.0/modules/event/enddate',
		'http://purl.org/rss/1.0/modules/event/location',
		'http://purl.org/rss/1.0/modules/event/organizer',
		'http://purl.org/rss/1.0/modules/event/type'))
	{
	}
	
	return;
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		}
	];
}

sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $triples = shift;
	my $subject = shift;
	my $S = $triples->{$subject};
	my @rv = (['BEGIN','VEVENT']);

	# UID
	my $uid = firstOf($S, 'urn:ietf:rfc:2445#uid',
		'http://www.w3.org/2002/12/cal/ical#uid',
		'http://purl.org/dc/terms/identifier',
		'http://purl.org/dc/elements/1.1/identifier');
	push @rv, ['UID', $uid]
		if ($uid);
	
	# Summary
	my $summary = firstOf($S, 'urn:ietf:rfc:2445#summary',
		'http://www.w3.org/2002/12/cal/ical#summary',
		'http://purl.org/rss/1.0/title',
		'urn:ietf:rfc:4287#title',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title');
	push @rv, ['SUMMARY', $summary]
		if ($summary);
	
	# Comment, Description
	my $desc = firstOf($S, 'urn:ietf:rfc:2445#comment',
		'http://www.w3.org/2002/12/cal/ical#comment',
		'http://purl.org/dc/terms/abstract');
	push @rv, ['COMMENT', $desc]
		if ($desc);
	my @desc = allOf($S, 'urn:ietf:rfc:2445#description',
		'http://www.w3.org/2002/12/cal/ical#description',
		'http://purl.org/rss/1.0/description',
		'urn:ietf:rfc:4287#content',
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description');
	foreach my $desc (@desc)
		{ push @rv, ['DESCRIPTION', $desc]; }
	
	# Location
	my $loc = firstOf($S, 'urn:ietf:rfc:2445#location',
		'http://www.w3.org/2002/12/cal/ical#location',
		'http://purl.org/rss/1.0/modules/event/location');
	push @rv, ['LOCATION', Cognition::Export::Location::outText($triples, $loc)]
		if ($loc);
		
	# Geo
	my @geos = allOfObj($S, 'urn:ietf:rfc:2445#geo',
		'http://www.w3.org/2002/12/cal/ical#location');
	foreach my $geo (@geos)
	{
		if ($geo->type && $triples->{$geo->toString})
		{
			my $geoSubject = $triples->{$geo->toString};
			if (Cognition::Export::Point::is($geoSubject))
			{
				my ($key, $val) = Cognition::Export::Point::outVCFLine($triples, $geo->toString);
				push @rv, [$key, $val, 1];
			}
		}
	}

	# Categories
	my @cats = allOfObj($S, 'urn:ietf:rfc:2445#categories',
		'http://www.w3.org/2002/12/cal/ical#categories',
		'http://purl.org/rss/1.0/modules/event/type');
	my $catstring = '';
	foreach my $cat (@cats)
	{
		if ($cat->type && $triples->{$cat->toString})
		{
			my $catSubject = $triples->{$cat->toString};
			my $category = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#tag',
				'http://purl.org/dc/terms/description',
				'http://purl.org/dc/elements/1.1/description');
			$catstring .= ','.uc($category)
				if (length $category);
		}
		else
		{
			my $category = $cat->toString;
			$category =~ s/(^\s+|\s+$)//g;
			$category =~ s/\s+/ /g;
			$catstring .= ','.uc($category);
		}
	}
	$catstring =~ s/^\,//;
	push @rv, ['CATEGORIES',$catstring,1]
		if ($catstring);

	# Resources
	@cats = allOfObj($S, 'urn:ietf:rfc:2445#resources',
		'http://www.w3.org/2002/12/cal/ical#resources');
	$catstring = '';
	foreach my $cat (@cats)
	{
		if ($cat->type && $triples->{$cat->toString})
		{
			my $catSubject = $triples->{$cat->toString};
			my $category = firstOf($catSubject,
				'http://microformats.org/wiki/rel-tag#tag',
				'http://purl.org/dc/terms/description',
				'http://purl.org/dc/elements/1.1/description');
			$catstring .= ','.uc($category)
				if (length $category);
		}
		else
		{
			my $category = $cat->toString;
			$category =~ s/(^\s+|\s+$)//g;
			$category =~ s/\s+/ /g;
			$catstring .= ','.uc($category);
		}
	}
	$catstring =~ s/^\,//;
	push @rv, ['RESOURCES',$catstring,1]
		if ($catstring);


#
# Still need to be implemented!
#
#	•	related-to (link) *
#

	# DTStart, DTEnd, DTStamp, Duration, Last-Modified, Created
	my $dt = firstOf($S, 'urn:ietf:rfc:2445#dtstart',
		'http://www.w3.org/2002/12/cal/ical#dtstart',
		'http://purl.org/rss/1.0/modules/event/startdate');
	push @rv, ['DTSTART', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#dtend',
		'http://www.w3.org/2002/12/cal/ical#dtend',
		'http://purl.org/rss/1.0/modules/event/enddate');
	push @rv, ['DTEND', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#dtstamp',
		'http://www.w3.org/2002/12/cal/ical#dtstamp',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	push @rv, ['DTSTAMP', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#duration',
		'http://www.w3.org/2002/12/cal/ical#duration');
	push @rv, ['DURATION', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#last-modified',
		'http://www.w3.org/2002/12/cal/ical#last-modified',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	push @rv, ['LAST-MODIFIED', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#created',
		'http://www.w3.org/2002/12/cal/ical#created',
		'http://purl.org/dc/terms/available',
		'http://purl.org/dc/terms/created');
	push @rv, ['CREATED', $dt] if ($dt);
	
	# Due, Completed - Technically only for VTODO, but it makes things much
	# easier if we just accept it for VEVENT too.
	$dt = firstOf($S, 'urn:ietf:rfc:2445#due',
		'http://www.w3.org/2002/12/cal/ical#due');
	push @rv, ['DUE', $dt] if ($dt);
	$dt = firstOf($S, 'urn:ietf:rfc:2445#completed',
		'http://www.w3.org/2002/12/cal/ical#completed');
	push @rv, ['COMPLETED', $dt] if ($dt);
	
	# RDATE, RRULE, EXDATE, EXRULE
	foreach my $p (@{['rdate', 'rrule', 'exdate', 'exrule']})
	{
		my @R = allOf($S, 'urn:ietf:rfc:2445#'.$p,
			'http://www.w3.org/2002/12/cal/ical#'.$p);
		foreach my $r (@R)
			{ push @rv, [uc($p), $r]; }
	}
	# Recurrence-ID
	my $rid = firstOf($S, 'urn:ietf:rfc:2445#recurrence-id',
		'http://www.w3.org/2002/12/cal/ical#recurrence-id');
	push @rv, ['RECURRENCE-ID', $rid]
		if ($rid);

	# URL
	my $url = firstOf($S, 'urn:ietf:rfc:2445#url',
		'http://www.w3.org/2002/12/cal/ical#url');
	push @rv, ['URL', $url]
		if ($url);
	
	# Nested alarms
	my @va = allOf($S, 'urn:ietf:rfc:2445#valarm',
		'http://www.w3.org/2002/12/cal/ical#valarm');
	foreach my $va (@va)
	{
		if (Cognition::Export::Alarm::is($triples->{$va}))
			{ push @rv, ['VALARM', Cognition::Export::Alarm::_outICS($triples, $va) ]; }
	}
	
	# Attach
	my @attachments = allOf($S, 'urn:ietf:rfc:2445#attach',
		'http://www.w3.org/2002/12/cal/ical#attach',
		'http://purl.org/dc/terms/hasPart');
	foreach my $att (@attachments)
	{
		if ($triples->{$att})
		{
			my $E = $triples->{$att};
			next unless ($E);
			next unless ($E->property('http://microformats.org/wiki/rel-enclosure#enclosure'));
			my $e = $E->firstVal('http://microformats.org/wiki/rel-enclosure#enclosure');
			push @rv, ['ATTACH;VALUE=uri',$e] if ($e);
		}
		else
		{
			push @rv, ['ATTACH;VALUE=uri',$att];
		}
	}

	# Class, Status, Transp
	my $class = firstOf($S, 'urn:ietf:rfc:2445#class',
		'http://www.w3.org/2002/12/cal/ical#class');
	push @rv, ['CLASS', uc($class)]
		if ($class);
	$class = firstOf($S, 'urn:ietf:rfc:2445#status',
		'http://www.w3.org/2002/12/cal/ical#status');
	push @rv, ['STATUS', uc($class)]
		if ($class);
	$class = firstOf($S, 'urn:ietf:rfc:2445#transp',
		'http://www.w3.org/2002/12/cal/ical#transp');
	push @rv, ['TRANSP', uc($class)]
		if ($class);
	$class = firstOf($S, 'urn:ietf:rfc:2445#priority',
		'http://www.w3.org/2002/12/cal/ical#priority');
	push @rv, ['PRIORITY', uc($class)]
		if ($class);

	# Sequence	
	my $seq = firstOf($S, 'urn:ietf:rfc:2445#sequence',
		'http://www.w3.org/2002/12/cal/ical#sequence');
	push @rv, ['SEQUENCE', $seq] if ($seq);

	# Percent Complete - Technically only for VTODO, but it makes things much
	# easier if we just accept it for VEVENT too.
	my $pc = firstOf($S, 'urn:ietf:rfc:2445#percent-complete',
		'http://www.w3.org/2002/12/cal/ical#percent-complete');
	if ($pc)
	{
		$pc = sprintf("%d%%", $pc);
		push @rv, ['PERCENT-COMPLETE', $pc];
	}
	
	# Attendee
	my @agents = allOfObj($S, 'urn:ietf:rfc:2445#attendee',
		'http://www.w3.org/2002/12/cal/ical#attendee');
	foreach my $agent (@agents)
	{
		if ($agent->type && $triples->{$agent->toString})
		{
			my $agentSubject = $triples->{$agent->toString};
			if (Cognition::Export::Contact::is($agentSubject))
			{
				my $a = Cognition::Export::Contact::outVCF($triples, $agent->toString);
				push @rv, ['ATTENDEE',$a];
			}
			else
			{
				my $a = firstOf($agentSubject,
					'urn:ietf:rfc:2426#fn',
					'http://www.w3.org/2006/vcard/ns#fn',
					'http://www.w3.org/2006/03/hcard#fn',
					'http://www.w3.org/2001/vcard-rdf/3.0#fn',
					'http://xmlns.com/foaf/0.1/fullName',
					'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
					'http://purl.org/dc/terms/title',
					'http://purl.org/dc/elements/1.1/title');
				push @rv, ['ATTENDEE;VALUE=text',$a] if ($a);
			}
		}
		else
		{
			my $a = $agent->toString;
			$a =~ s/(^\s+|\s+$)//g;
			$a =~ s/\s+/ /g;
			push @rv, ['ATTENDEE;VALUE=text',$a] if ($a);
		}
	}
		
	# contacts
	my @agents = allOfObj($S, 'urn:ietf:rfc:2445#contact',
		'http://www.w3.org/2002/12/cal/ical#contact');
	foreach my $agent (@agents)
	{
		if ($agent->type && $triples->{$agent->toString})
		{
			my $agentSubject = $triples->{$agent->toString};
			if (Cognition::Export::Contact::is($agentSubject))
			{
				my $a = Cognition::Export::Contact::outVCF($triples, $agent->toString);
				push @rv, ['CONTACT',$a];
			}
			else
			{
				my $a = firstOf($agentSubject,
					'urn:ietf:rfc:2426#fn',
					'http://www.w3.org/2006/vcard/ns#fn',
					'http://www.w3.org/2006/03/hcard#fn',
					'http://www.w3.org/2001/vcard-rdf/3.0#fn',
					'http://xmlns.com/foaf/0.1/fullName',
					'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
					'http://purl.org/dc/terms/title',
					'http://purl.org/dc/elements/1.1/title');
				push @rv, ['CONTACT;VALUE=text',$a] if ($a);
			}
		}
		else
		{
			my $a = $agent->toString;
			$a =~ s/(^\s+|\s+$)//g;
			$a =~ s/\s+/ /g;
			push @rv, ['CONTACT;VALUE=text',$a] if ($a);
		}
	}
		
	# orgs
	my @agents = allOfObj($S, 'urn:ietf:rfc:2445#organizer',
		'http://www.w3.org/2002/12/cal/ical#organizer',
		'http://purl.org/rss/1.0/modules/event/organizer');
	foreach my $agent (@agents)
	{
		if ($agent->type && $triples->{$agent->toString})
		{
			my $agentSubject = $triples->{$agent->toString};
			if (Cognition::Export::Contact::is($agentSubject))
			{
				my $a = Cognition::Export::Contact::outVCF($triples, $agent->toString);
				push @rv, ['ORGANIZER',$a];
			}
			else
			{
				my $a = firstOf($agentSubject,
					'urn:ietf:rfc:2426#fn',
					'http://www.w3.org/2006/vcard/ns#fn',
					'http://www.w3.org/2006/03/hcard#fn',
					'http://www.w3.org/2001/vcard-rdf/3.0#fn',
					'http://xmlns.com/foaf/0.1/fullName',
					'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
					'http://purl.org/dc/terms/title',
					'http://purl.org/dc/elements/1.1/title');
				push @rv, ['ORGANIZER;VALUE=text',$a] if ($a);
			}
		}
		else
		{
			my $a = $agent->toString;
			$a =~ s/(^\s+|\s+$)//g;
			$a =~ s/\s+/ /g;
			push @rv, ['ORGANIZER;VALUE=text',$a] if ($a);
		}
	}
		
	# Done.
	push @rv, ['END','VEVENT'];
	
	# Convert to text/directory syntax.
	my $rv = '';
	foreach my $line (@rv)
	{
		my ($k, $v, $soft) = @{$line};
		
		if ($k =~ /VALUE.uri$/i && $v =~ /^data\:/)
		{
			my $uri = URI->new($v);
			$v = encode_base64($uri->data, '');
			$k =~ s/VALUE.uri$/ENCODING\=b/i;
		}
		
		$v =~ s/\\/\\\\/g;
		$v =~ s/\r?\n/\\n/g;
		unless ($soft)
		{
			$v =~ s/\;/\\\;/g;
			$v =~ s/\,/\\\,/g;
		}
		
		$rv .= "$k:$v\r\n";
	}
	return $rv;
}

1;
