#!/usr/bin/perl

######################################################################
package Cognition::uF::hAlarm;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub MAX_INCLUDE_ITERATIONS { 6; }


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('valarm', $within);
	my @rv;
	
	foreach my $ev (@nodes)
	{
		push @rv, parse($page, $ev);
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	if (length $this->{summary})
	{
		push @rv, 'urn:x-valarm:' . CGI::Util::escape($this->{summary});
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2445#';
	my $CLASS   = shift;

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'valarm', 1));
	
	# Patterns for simple data.
	my @_SINGLES  = qw(summary trigger description duration action repeat);
	my @_SINGLESR = qw();
	my @_MULTIS   = qw();
	my @_MULTISR  = qw();
	my @_OBJECTS  = qw(attendee attach);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal($ns.$property, $val);
	}

	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V)
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return { 'ICAL' => { nsurl=>'urn:ietf:rfc:2445#',title=>'iCalendar' } };
} #/sub metadata_ns


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $event_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $event_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($event_node, $page) 
		              + Cognition::uF::ufp_include2($event_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($event_node, $page);
	Cognition::uF::ufp_table($event_node, $page);

	# Check nested hcards to see if they have a 'useful' class, and if so, parse them in.
	my @nested = Cognition::HTMLParser::Utils::searchClass('vcard', $event_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) attendee (\s|$) /x)
		{
			push @{ $rv->{attendee} }, Cognition::uF::hCard::parse($page, $h);
		}
	}
	
	# Enclosures
	my @encs = Cognition::uF::RelEnclosure::parse_all($page, $event_node);
	$rv->{attach} = \@encs unless (!@encs);	
	
	# Now that we have reached here, no other composite microformats have
	# any business being nested within this root element. So in the interest
	# of Microformat opacity, let's destroy them.
	Cognition::uF::destroyer($event_node);
	
	# Simple attributes - singles
	my @simples = qw(summary trigger description duration action repeat);
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		my $value_node  = $value_nodes[0];
		next unless ($value_node);
		my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);
		$rv->{$simple} = $value;
	}	

	# Simple attributes - possible duplicates
	my @simples = qw(attendee attach);
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		foreach my $value_node (@value_nodes)
		{
			next unless ($value_node);
			my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);
			
			if ($simple eq 'attach')
			{
				if ($value_node->localname eq 'a' || $value_node->localname eq 'area')
					{ $value = $page->abs_url($value_node->getAttribute('href')); }
				elsif ($value_node->localname eq 'img')
					{ $value = $page->abs_url($value_node->getAttribute('src')); }
				elsif ($value_node->localname eq 'object')
					{ $value = $page->abs_url($value_node->getAttribute('data')); }
			}
			
			push @{$rv->{$simple}}, $value;
		}
	}	
	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::hEvent




