#!/usr/bin/perl

######################################################################
package Cognition::uF::hEntry;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Data::Dumper;

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub MAX_INCLUDE_ITERATIONS { 6; }


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes1 = Cognition::HTMLParser::Utils::searchClass('hentry', $within);
	my @nodes2 = Cognition::HTMLParser::Utils::searchClass('hslice', $within);
	my @nodes = (@nodes1, @nodes2);
	my @rv;
	
	foreach my $e (@nodes)
	{
		my $E = parse($page, $e);
		push @rv, $E;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;

	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:4287#';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'entry', 1));
	
	# Patterns for simple data.
	my @_SINGLES  = qw(title content summary published);
	my @_SINGLESR = qw(link);
	my @_MULTIS   = qw(updated);
	my @_MULTISR  = qw();
	my @_OBJECTS  = qw(author category enclosure);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal($ns.$property, $val);
	}

	# Resource-type properties which exist as singles.
	foreach my $property (@_SINGLESR)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property}, 1);
		$subject->addVal($ns.$property, $val);
	}
		
	# Properties which exist as multiple values.
	foreach my $property (@_MULTIS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	# Resource-type properties which exist as multiple values.
	foreach my $property (@_MULTISR)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V, 1);
			$subject->addVal($ns.$property, $val);
		}
	}

	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V)
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return { 'atom' => { nsurl=>'urn:ietf:rfc:4287#',title=>'Atom' } };
} #/sub namespace


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	bless $rv;
	
	# hAtom spec: an Entry element is identified by class name hentry
	my $root_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $root_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($root_node, $page) 
		              + Cognition::uF::ufp_include2($root_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($root_node, $page);
	Cognition::uF::ufp_table($root_node, $page);

	# Author
	my @nested = Cognition::HTMLParser::Utils::searchClass('vcard', $root_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) author (\s|$) /x)
		{
			push @{ $rv->{author} }, Cognition::uF::hCard::parse($page, $h);
		}
	}	
	if (!defined $rv->{author})
	{
		foreach my $h (@nested)
		{
			if ($h->tagName == 'address')
			{
				push @{ $rv->{author} }, Cognition::uF::hCard::parse($page, $h);
			}
		}
		if (!defined $rv->{author})
		{
			foreach my $address ($root_node->getElementsByTagName('address'))
			{
				push @{ $rv->{author} }, Cognition::uF::hCard::parse_all($page, $address);
			}
		}
	}
	
	# Now that we have reached here, no other composite microformats have
	# any business being nested within this root element. So in the interest
	# of Microformat opacity, let's destroy them.
	Cognition::uF::destroyer($root_node);
	
	# Categories
	my @tags = Cognition::uF::RelTag::parse_all($page, $root_node);
	$rv->{category} = \@tags unless (!@tags);
	
	# Enclosure
	my @encs = Cognition::uF::RelEnclosure::parse_all($page, $root_node);
	$rv->{enclosure} = \@encs unless (!@encs);
	
	# Entry title
	my @nodes = Cognition::HTMLParser::Utils::searchClass('entry-title', $root_node);
	if (@nodes)
	{
		$rv->{title} = Cognition::HTMLParser::Utils::STRINGIFY($nodes[0]);
	}
	else
	{
		foreach my $tag ($root_node->getElementsByTagName('*'))
		{
			if ($tag->tagName =~ /^h[1-9]?$/i)
			{
				$rv->{title} = Cognition::HTMLParser::Utils::STRINGIFY($tag);
				last;
			}
		}
	}
	if (!defined $rv->{title})
	{
		# Should fetch <title> here from $page unless $page contains an element
		# of class 'hfeed'.
	}
	
	# Entry content
	foreach my $x (Cognition::HTMLParser::Utils::searchClass('entry-content', $root_node))
	{
		$rv->{content} .= Cognition::HTMLParser::Utils::STRINGIFY($x)." ";
	}
	$rv->{content} =~ s/ $//;
	
	# Entry summary
	foreach my $x (Cognition::HTMLParser::Utils::searchClass('entry-summary', $root_node))
	{
		$rv->{summary} .= Cognition::HTMLParser::Utils::STRINGIFY($x)." ";
	}
	$rv->{summary} =~ s/ $//;
	
	# Updated
	foreach my $x (Cognition::HTMLParser::Utils::searchClass('updated', $root_node))
	{
		my $X = Cognition::HTMLParser::Utils::STRINGIFY($x);
		push @{$rv->{updated}}, Cognition::uF::datetime($X);
	}

	# Published
	my @nodes = Cognition::HTMLParser::Utils::searchClass('published', $root_node);
	if (@nodes)
	{
		my $X = Cognition::HTMLParser::Utils::STRINGIFY($nodes[0]);
		$rv->{published} = Cognition::uF::datetime($X);
	}
	
	# Bookmark
	my @nodes = Cognition::HTMLParser::Utils::searchRel('bookmark', $root_node);
	if (@nodes)
	{
		$rv->{link} = $page->abs_url($nodes[0]->getAttribute('href'));
	}
	else
	{
		$rv->{link} = $rv->uri;
	}
	
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::hEntry

